@@ -1,6 +1,6 @@
 //! # map_vec: Map and Set APIs backed by Vecs.
 #![cfg_attr(not(test), no_std)]
-#![feature(drain_filter, shrink_to, try_reserve)]
+#![cfg_attr(feature = "nightly", feature(drain_filter, shrink_to, try_reserve))]
 
 extern crate alloc;
 
