@@ -11,12 +11,10 @@ use core::{
 /// ## Example
 ///
 /// ```
-/// fn main() {
-///   let mut map = map_vec::Map::new();
-///   map.insert("hello".to_string(), "world".to_string());
-///   map.entry("hello".to_string()).and_modify(|mut v| v.push_str("!"));
-///   assert_eq!(map.get("hello").map(String::as_str), Some("world!"))
-/// }
+/// let mut map = map_vec::Map::new();
+/// map.insert("hello".to_string(), "world".to_string());
+/// map.entry("hello".to_string()).and_modify(|mut v| v.push_str("!"));
+/// assert_eq!(map.get("hello").map(String::as_str), Some("world!"))
 /// ```
 #[derive(Clone, Default, PartialEq, Eq)]
 pub struct Map<K, V> {
@@ -157,26 +155,27 @@ impl<K: Eq, V> Map<K, V> {
     }
 
     pub fn reserve(&mut self, additional: usize) {
-        self.backing.reserve(additional)
+        self.backing.reserve(additional);
     }
 
+    #[cfg(not(feature = "nightly"))]
+    pub fn retain(&mut self, mut f: impl FnMut(&K, &mut V) -> bool) {
+        let mut retained = Vec::new();
+        for (k, mut v) in self.backing.drain(..) {
+            if f(&k, &mut v) {
+                retained.push((k, v))
+            }
+        }
+        self.backing = retained;
+    }
+
+    #[cfg(feature = "nightly")]
     pub fn retain(&mut self, mut f: impl FnMut(&K, &mut V) -> bool) {
         self.backing.drain_filter(|(k, ref mut v)| !f(k, v));
     }
 
-    pub fn shrink_to(&mut self, min_capacity: usize) {
-        self.backing.shrink_to(min_capacity)
-    }
-
     pub fn shrink_to_fit(&mut self) {
-        self.backing.shrink_to_fit()
-    }
-
-    pub fn try_reserve(
-        &mut self,
-        additional: usize,
-    ) -> Result<(), alloc::collections::TryReserveError> {
-        self.backing.try_reserve(additional)
+        self.backing.shrink_to_fit();
     }
 
     pub fn values(&self) -> impl Iterator<Item = &V> + DoubleEndedIterator + ExactSizeIterator {
@@ -190,6 +189,20 @@ impl<K: Eq, V> Map<K, V> {
     }
 }
 
+#[cfg(feature = "nightly")]
+impl<K: Eq, V> Map<K, V> {
+    pub fn shrink_to(&mut self, min_capacity: usize) {
+        self.backing.shrink_to(min_capacity)
+    }
+
+    pub fn try_reserve(
+        &mut self,
+        additional: usize,
+    ) -> Result<(), alloc::collections::TryReserveError> {
+        self.backing.try_reserve(additional)
+    }
+}
+
 impl<K: Debug, V: Debug> fmt::Debug for Map<K, V> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_map()
@@ -353,6 +366,93 @@ impl<'a, K: 'a, V: 'a> VacantEntry<'a, K, V> {
     }
 }
 
+#[cfg(feature = "serde")]
+mod map_serde {
+    use super::*;
+
+    use core::marker::PhantomData;
+    use serde::{
+        de::{Deserialize, Deserializer, MapAccess, Visitor},
+        ser::{Serialize, SerializeMap, Serializer},
+    };
+
+    impl<K, V> Serialize for Map<K, V>
+    where
+        K: Serialize + Eq,
+        V: Serialize,
+    {
+        fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+        where
+            S: Serializer,
+        {
+            let mut map = serializer.serialize_map(Some(self.len()))?;
+            for (k, v) in self {
+                map.serialize_entry(k, v)?;
+            }
+            map.end()
+        }
+    }
+
+    impl<'de, K, V> Deserialize<'de> for Map<K, V>
+    where
+        K: Deserialize<'de> + Eq,
+        V: Deserialize<'de>,
+    {
+        fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+        where
+            D: Deserializer<'de>,
+        {
+            struct MapVisitor<K, V> {
+                marker: PhantomData<fn() -> Map<K, V>>,
+            }
+
+            impl<'de, K, V> Visitor<'de> for MapVisitor<K, V>
+            where
+                K: Deserialize<'de> + Eq,
+                V: Deserialize<'de>,
+            {
+                type Value = Map<K, V>;
+
+                fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                    formatter.write_str("a map")
+                }
+
+                fn visit_map<M>(self, mut access: M) -> Result<Self::Value, M::Error>
+                where
+                    M: MapAccess<'de>,
+                {
+                    let mut map = Map::with_capacity(access.size_hint().unwrap_or(0));
+
+                    while let Some((key, value)) = access.next_entry()? {
+                        map.insert(key, value);
+                    }
+
+                    Ok(map)
+                }
+            }
+
+            deserializer.deserialize_map(MapVisitor {
+                marker: PhantomData,
+            })
+        }
+    }
+
+    #[cfg(test)]
+    #[test]
+    fn test_serde() {
+        let mut m = Map::new();
+        m.insert("onefish", "twofish");
+        m.insert("redfish", "bluefish");
+        let json = serde_json::to_string(&m).unwrap();
+        assert_eq!(
+            json.as_str(),
+            r#"{"onefish":"twofish","redfish":"bluefish"}"#
+        );
+        let m2: Map<&str, &str> = serde_json::from_str(&json).unwrap();
+        assert_eq!(m2, m);
+    }
+}
+
 // taken from libstd/collections/hash/map.rs @ 7454b2
 #[cfg(test)]
 mod test_map {
@@ -361,7 +461,7 @@ mod test_map {
         Map,
     };
 
-    use std::{cell::RefCell, collections::CollectionAllocErr::*, usize};
+    use std::{cell::RefCell, usize};
 
     use rand::{thread_rng, Rng};
 
@@ -1140,90 +1240,25 @@ mod test_map {
         assert_eq!(map[&6], 60);
     }
 
+    #[cfg(feature = "nightly")]
     #[test]
     fn test_try_reserve() {
         let mut empty_bytes: Map<u8, u8> = Map::new();
 
         const MAX_USIZE: usize = usize::MAX;
 
-        if let Err(CapacityOverflow) = empty_bytes.try_reserve(MAX_USIZE) {
+        if let Err(std::collections::TryReserveError::CapacityOverflow) =
+            empty_bytes.try_reserve(MAX_USIZE)
+        {
         } else {
             panic!("usize::MAX should trigger an overflow!");
         }
 
-        if let Err(AllocErr) = empty_bytes.try_reserve(MAX_USIZE / 8) {
+        if let Err(std::collections::TryReserveError::AllocError { .. }) =
+            empty_bytes.try_reserve(MAX_USIZE / 8)
+        {
         } else {
             panic!("usize::MAX / 8 should trigger an OOM!")
         }
     }
 }
-
-#[cfg(feature = "serde")]
-const _IMPL_SERDE_FOR_MAP: () = {
-    use core::marker::PhantomData;
-    use serde::{
-        de::{Deserialize, Deserializer, MapAccess, Visitor},
-        ser::{Serialize, SerializeMap, Serializer},
-    };
-
-    impl<K, V> Serialize for Map<K, V>
-    where
-        K: Serialize + Eq,
-        V: Serialize,
-    {
-        fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-        where
-            S: Serializer,
-        {
-            let mut map = serializer.serialize_map(Some(self.len()))?;
-            for (k, v) in self {
-                map.serialize_entry(k, v)?;
-            }
-            map.end()
-        }
-    }
-
-    impl<'de, K, V> Deserialize<'de> for Map<K, V>
-    where
-        K: Deserialize<'de> + Eq,
-        V: Deserialize<'de>,
-    {
-        fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-        where
-            D: Deserializer<'de>,
-        {
-            struct MapVisitor<K, V> {
-                marker: PhantomData<fn() -> Map<K, V>>,
-            }
-
-            impl<'de, K, V> Visitor<'de> for MapVisitor<K, V>
-            where
-                K: Deserialize<'de> + Eq,
-                V: Deserialize<'de>,
-            {
-                type Value = Map<K, V>;
-
-                fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
-                    formatter.write_str("a map")
-                }
-
-                fn visit_map<M>(self, mut access: M) -> Result<Self::Value, M::Error>
-                where
-                    M: MapAccess<'de>,
-                {
-                    let mut map = Map::with_capacity(access.size_hint().unwrap_or(0));
-
-                    while let Some((key, value)) = access.next_entry()? {
-                        map.insert(key, value);
-                    }
-
-                    Ok(map)
-                }
-            }
-
-            deserializer.deserialize_map(MapVisitor {
-                marker: PhantomData,
-            })
-        }
-    }
-};
